import json

from django.db.models import Avg, Sum, Count
from pypinyin import pinyin, Style

from config.config import TABLE_PRE
from inspect_report.agency import get_team_names, get_agent_name
from inspect_report.cron import get_remain
from inspect_report.models import ScoreStat, Round, SeatStat, Seat, Country, Tasks, CheckSession, RulesStat, Team
from inspect_report.pagehelper import page_with_seat_name


def set_city_country_by_team_name(objects, key):
    team_country_dict = get_city_country_team()

    for object_dict in objects:
        object_dict['city'] = team_country_dict[object_dict[key]]['city']
        object_dict['country'] = team_country_dict[object_dict[key]]['country']
    return objects


def get_city_country_team():
    teams = Team.objects.values('name', 'country_id')
    countries = Country.objects.values('id', 'name', 'parent')
    countries_dict = dict()
    for country in countries:
        countries_dict[country['id']] = {'name': country['name'], 'parent': country['parent']}
    teams_dict = dict()
    for team in teams:
        teams_dict[team['name']] = dict()
        country_dict = countries_dict[team['country_id']]
        if country_dict['parent'] is not None and int(country_dict['parent']) != 153:
            teams_dict[team['name']]['country'] = country_dict['name']
            teams_dict[team['name']]['city'] = countries_dict[country_dict['parent']]['name']
        else:
            teams_dict[team['name']]['country'] = ''
            teams_dict[team['name']]['city'] = country_dict['name']
    return teams_dict


def db_static_score(req):
    city_id = req.data.get('city', None)
    country_id = req.data.get('country', None)
    task_condition = req.data.get('date_condition', {})

    team_names = get_team_names(city_id, country_id)
    task_condition['task__in'] = team_names

    sort = req.data.get('sort', '-avg_score')

    all_scores = ScoreStat.objects.filter(**task_condition).extra(select={'area': "task"}).values('area').annotate(
        avg_score_svc=Round(Avg('service_score'), 2), avg_score_bus=Round(Avg('business_score'), 2),
        avg_score=Round(Avg('score'), 2), total_session=Sum('total_session'), validate_num=Sum('validate_num'),
        validate_zero=Sum('validate_zero')).order_by(sort)
    return all_scores


def db_seat_rule(req, if_page=False):
    city_id = req.data.get('city', None)
    country_id = req.data.get('country', None)
    team_id = req.data.get('team', None)
    agent_name = req.data.get('seat', '')
    sort = req.data.get('sort', 'des')
    task_condition = req.data.get('date_condition', {})

    team_names = get_team_names(city_id, country_id, team_id)

    task_condition['task__in'] = team_names

    if agent_name != '' and agent_name is not None:
        task_condition['agentName'] = agent_name
    if sort == 'des':
        all_seats = SeatStat.objects.filter(**task_condition).extra(select={'agentName': 'agentName'}) \
            .values('agentName').annotate(remainTime=Sum('remainTime'), validate_session=Sum('validate_session'),
                                          total_session=Sum('total_session')).order_by('-validate_session')

    else:
        all_seats = SeatStat.objects.filter(**task_condition).extra(select={'agentName': 'agentName'}) \
            .values('agentName').annotate(remainTime=Sum('remainTime'), validate_session=Sum('validate_session'),
                                          total_session=Sum('total_session')).order_by('validate_session')

    if if_page:
        page = req.data.get('page', '1')
        page_size = req.data.get('page_size', '10')
        total_count, seats = page_with_seat_name(all_seats, page, page_size, agent=False)
    else:
        page = 1
        page_size = 10
        total_count = 0
        seats = all_seats

    return_data = []
    agent_name_set = set()
    for s in seats:
        detail = {'agentName': s['agentName'], 'validate_session': s['validate_session'],
                  'remainTime': s['remainTime'], 'total_session': s['total_session'],
                  'ratio': round(s['validate_session'] / s['total_session'], 2)}
        agent_name_set.add(s['agentName'])
        return_data.append(detail)
    seat_code_name_lists = Seat.objects.filter(code__in=agent_name_set).values('code', 'name')
    seat_code_name_dict = dict()
    for seat_code_name in seat_code_name_lists:
        seat_code_name_dict[seat_code_name['code']] = seat_code_name['name']
    if sort == 'asc':
        data_sort = sorted(return_data, key=lambda x: x['ratio'])
        for i in range(len(data_sort)):
            if if_page:
                data_sort[i]['ranking'] = total_count - (int(page) - 1) * int(page_size) - i
            data_sort[i]['ratio'] = '{0}%'.format(int(data_sort[i]['ratio'] * 100))
            data_sort[i]['seatName'] = seat_code_name_dict.get(data_sort[i]['agentName'], data_sort[i]['agentName'])
    else:
        data_sort = sorted(return_data, key=lambda x: x['ratio'], reverse=True)
        for i in range(len(data_sort)):
            if if_page:
                data_sort[i]['ranking'] = (int(page) - 1) * int(page_size) + i + 1
            data_sort[i]['ratio'] = '{0}%'.format(int(data_sort[i]['ratio'] * 100))
            data_sort[i]['seatName'] = seat_code_name_dict.get(data_sort[i]['agentName'], data_sort[i]['agentName'])
    return total_count, data_sort


def db_seat_check(req, if_page=False):
    city_id = req.data.get('city', '')
    country_id = req.data.get('country', '')
    team_id = req.data.get('team', '')
    agent_name = req.data.get('agentName', '')
    task_condition = req.data.get('at_condition', {})
    task_condition['hasCheck'] = 1

    if city_id:
        city = Country.objects.filter(id=city_id).first()
        if city.parent is not None:
            task_condition['name__startswith'] = city.name
    print(task_condition)
    tasks = Tasks.objects.filter(**task_condition).values('id', 'name', 'sessionCollectionId')
    task_dict = {}
    task_name_dict = {}
    for t in tasks:
        task_name_dict[t['id']] = t['name']
        if t['sessionCollectionId'] in task_dict.keys():
            task_dict[t['sessionCollectionId']].append(t['id'])
        else:
            task_dict[t['sessionCollectionId']] = [t['id'], ]
    return_data = []
    total_count = 0

    agent_names = get_agent_name(city_id, country_id, team_id, agent_name)

    if len(agent_names) > 0:
        seat_code_name_lists = Seat.objects.filter(code__in=agent_names).values('code', 'name')
    else:
        seat_code_name_lists = Seat.objects.filter().values('code', 'name')
    seat_code_name_dict = dict()
    for seat_code_name in seat_code_name_lists:
        seat_code_name_dict[seat_code_name['code']] = seat_code_name['name']

    for k, v in task_dict.items():
        # session_collection = t.sessionCollectionId
        table_name = TABLE_PRE + k
        tn = CheckSession.set_table(table_name)
        session_condition = {'taskId__in': v}
        if len(agent_names) > 0:
            session_condition['agentName__in'] = list(seat_code_name_dict.keys())
        print(session_condition)
        all_checks = tn.objects.filter(**session_condition).values('id', 'createdAt', 'agentName', 'extra', 'taskId',
                                                                   'remainTime', 'startTime', 'closeTime',
                                                                   'scoreItemRecord', 'sessionId')

        if if_page:
            page = req.data.get('page', '1')
            page_size = req.data.get('page_size', '10')
            total_count, checks = page_with_seat_name(all_checks, page, page_size, agent=False)
        else:
            total_count = 0
            checks = all_checks

        for check in checks:
            result = check['scoreItemRecord']
            t_name = task_name_dict[check['taskId']]
            total_score = 100
            extra = json.loads(check['extra'])
            remain = get_remain(check)
            f_name = extra['filename'] if extra and 'filename' in extra.keys() else ''
            detail = {'id': check['id'], 'sessionId': check['sessionId'], 'createdAt': check['createdAt'],
                      'area': t_name.split('_')[0],
                      'seatName': seat_code_name_dict.get(check['agentName'], check['agentName']),
                      'agentName': check['agentName'], 'filename': f_name,
                      'remainTime': remain, 'sessionCollectionId': k, 'taskId': check['taskId']}
            rule_detail = {'fwyy': 15, 'kcb': 5, 'jsy': 3, 'fwtd': 5, 'yssb': 5, 'jysb': 5, 'yycl': 10,
                           'ywcc': 10, 'yyjlz': 10, 'fwjs': 10, 'wtgbj': 2, 'cgjxxhs': 20, 'wgzlf': 100, 'mgc': 100}
            if result:
                data = json.loads(result)
                for d in data:
                    if 'scoreItemName' in d.keys() and 'score' in d.keys():
                        item_score = 0 if d['score'] and d['score'] > 0 else 1
                        name_py = pinyin(d['scoreItemName'], style=Style.FIRST_LETTER, strict=False)
                        item_name = ''.join([x[0] for x in name_py])
                        # detail[item_name] = {d['scoreItemName']: item_score}
                        if item_name in rule_detail.keys():
                            if item_score == 0:
                                total_score -= rule_detail[item_name]
                            rule_detail[item_name] = item_score
            for kk, vv in rule_detail.items():
                if vv > 1:
                    rule_detail[kk] = 1
            if rule_detail['wgzlf'] == 0 or rule_detail['mgc'] == 0:
                total_score = 0
            detail.update(rule_detail)
            detail['total_score'] = total_score
            return_data.append(detail)
    return total_count, return_data


def db_seat_area(req):
    city_id = req.data.get('city', None)
    country_id = req.data.get('country', None)
    team_id = req.data.get('team', None)
    agent_name = req.data.get('seat', '')
    sort = req.data.get('sort', '-avg_score')
    task_condition = req.data.get('date_condition', '')

    team_names = get_team_names(city_id, country_id, team_id)

    task_condition['task__in'] = team_names

    if agent_name:
        task_condition['agentName'] = agent_name
    # , 'total_session', 'validate_num', 'validate_session', 'validate_zero'
    all_scores = ScoreStat.objects.filter(**task_condition).extra(
        select={'area': "task", 'agentName': 'agentName'}).values('area', 'agentName').annotate(
        avg_score_svc=Round(Avg('service_score'), 2),
        avg_score_bus=Round(Avg('business_score'), 2),
        avg_score=Round(Avg('score'), 2), total_session=Sum('total_session'),
        validate_num=Sum('validate_num'), validate_zero=Sum('validate_zero')).order_by(sort)

    return all_scores


def db_static_rule(req):
    city_id = req.data.get('city', None)
    country_id = req.data.get('country', None)
    team_id = req.data.get('team', None)

    team_names = get_team_names(city_id, country_id, team_id)

    task_condition = req.data.get('date_condition', {})

    task_condition['task__in'] = team_names

    call_count = SeatStat.objects.filter(**task_condition).aggregate(Sum("validate_session"))[
        'validate_session__sum']

    # tasks = RulesStat.objects.filter(**task_condition).values('rule', 'sessionCollectionId', 'rule_num')
    tasks = RulesStat.objects.all().filter(**task_condition).extra(
        select={'rule': "rule",
                'sessionCollectionId': 'sessionCollectionId'}).values('rule',
                                                                      'sessionCollectionId').annotate(
        rule_num=Count('id')).order_by('-rule_num')
    return_data = {}
    for t in tasks:
        name = t['rule']
        if name in return_data.keys():
            return_data[name]['count'] += int(t['rule_num'])
        else:
            return_data[name] = {'count': int(t['rule_num']), 'tables': []}
        if t['sessionCollectionId'] not in return_data[name]['tables']:
            return_data[name]['tables'].append(t['sessionCollectionId'])
    data_sort = []
    for k, v in return_data.items():
        v['rule'] = k
        v['ratio'] = round(v['count'] / call_count, 2) if call_count else 0
        v['ratio'] = '{0}%'.format(int(v['ratio'] * 100))
        v['call_count'] = call_count
        data_sort.append(v)

    return data_sort
